% HistogramDatabase
% A simple image database based on histogram comparison
% all images must be in subfolder IMAGES
% input: query: image; 
% comparison: vector comparison-function (see local function below)
function histogramDatabase(query, comparison)
show = 1;

% read all image-names
images = getFileNames('IMAGES');

% retrieve feature vector for all image-files
for i=1:length(images)
    img = imread(images{i});
    FEATURES(i,:)=getFeatureVector(img);
end

% retrieve vector for query image
qfv = getFeatureVector(query);

% print result 
showVectors(qfv,FEATURES,images)

% vector comparison
% (the loop could be avoided here,
%  but it's easier to understand this way)
for i = 1:length(images)
    C(i) = vectorComparison(qfv , FEATURES(i,:),comparison);
end

% sort the comparison
[top index] = sort(C)

% show the result
figure;
showResult(top, index, query, images);


return


%----------------------------------------
% local function: 
% get filenames of image (.jpg) files in 
% specified folder
function images = getFileNames(dirName)

d=dir(dirName);
nameCount = 0;
for i=1:length(d)
    filename = d(i).name;
    if length(filename)>4 & isequal(filename(end-3:end),'.jpg')
        nameCount = nameCount +1;
        images{nameCount}=[dirName '/' filename];
    end
end

%----------------------------------------
% local function: 
% get feature vector for image
function featureVector = getFeatureVector(img)

    % compute features
    
    % histogram
    featureVector = imhist(img,20);

    % the picture itself !
%     img = imresize(img,[16,16]);
%     featureVector = reshape(img,[1,256]);
    
    
    % always return feature vector as column-vector
    featureVector = reshape(featureVector,[1 length(featureVector)]);
    

%----------------------------------------
% local function
% show all feature-vectors (max. 9 pictures)
function showVectors(qfv,FEATURES,images)
    subplot(3,3,1);
    bar(qfv);
    title('QUERY');
    
    for i=1:length(images)
        subplot(3,3,i+1);
        if i+1 < 10
            bar(FEATURES(i,:));
            title(images{i});
        end
    end
    


%----------------------------------------
% local function
% vectorComparison
% you can choose between:
% intersection, cosine-distance, euclidean-distance
function c = vectorComparison(query,target, comparison);
    query  = double(query);
    target = double(target);

    % 1: intersection, 2: cosine, 3: euclidean
    switch comparison
        case 1 
            fprintf('Intersection\n');
            c = 1 - sum(min(query,target)) / sum(query);
        case 2 
            fprintf('Cosine-Distance\n');
            l1 = sqrt(query*query');
            l2 = sqrt(target*target');
            c = 1 - query * target' / (l1 * l2);
        case 3 
            fprintf('Euclidean\n');
            c = sqrt(sum((query - target).^2));
    end
    
   
%----------------------------------------
% local function
% show result (max. 9 pictures)
function showResult(top, index, query, images)
    subplot(3,3,1);
    imshow(query);
    title('QUERY');
    
    for i=1:length(images)
        ind = index(i); 
        subplot(3,3,i+1);
        if i+1 < 10
            img=imread(images{ind});
            imshow(img);
            ttl = sprintf('%s: %f',images{ind},top(i));
            title(ttl);
        end
    end

 